@@ -132,12 +132,8 @@ pub type Result<T> = result::Result<T, Error>;
 pub struct Error(imp::Error);
 
 impl error::Error for Error {
-    fn description(&self) -> &str {
-        error::Error::description(&self.0)
-    }
-
-    fn cause(&self) -> Option<&error::Error> {
-        error::Error::cause(&self.0)
+    fn source(&self) -> Option<&(dyn error::Error + 'static)> {
+        error::Error::source(&self.0)
     }
 }
 
@@ -163,6 +159,7 @@ impl From<imp::Error> for Error {
 ///
 /// An identity is an X509 certificate along with its corresponding private key and chain of certificates to a trusted
 /// root.
+#[derive(Clone)]
 pub struct Identity(imp::Identity);
 
 impl Identity {
@@ -220,10 +217,7 @@ where
     }
 }
 
-impl<S> MidHandshakeTlsStream<S>
-where
-    S: io::Read + io::Write,
-{
+impl<S> MidHandshakeTlsStream<S> {
     /// Returns a shared reference to the inner stream.
     pub fn get_ref(&self) -> &S {
         self.0.get_ref()
@@ -233,7 +227,12 @@ where
     pub fn get_mut(&mut self) -> &mut S {
         self.0.get_mut()
     }
+}
 
+impl<S> MidHandshakeTlsStream<S>
+where
+    S: io::Read + io::Write,
+{
     /// Restarts the handshake process.
     ///
     /// If the handshake completes successfully then the negotiated stream is
@@ -268,11 +267,7 @@ impl<S> error::Error for HandshakeError<S>
 where
     S: Any + fmt::Debug,
 {
-    fn description(&self) -> &str {
-        "handshake error"
-    }
-
-    fn cause(&self) -> Option<&error::Error> {
+    fn source(&self) -> Option<&(dyn error::Error + 'static)> {
         match *self {
             HandshakeError::Failure(ref e) => Some(e),
             HandshakeError::WouldBlock(_) => None,
@@ -608,7 +603,7 @@ impl<S: fmt::Debug> fmt::Debug for TlsStream<S> {
     }
 }
 
-impl<S: io::Read + io::Write> TlsStream<S> {
+impl<S> TlsStream<S> {
     /// Returns a shared reference to the inner stream.
     pub fn get_ref(&self) -> &S {
         self.0.get_ref()
@@ -618,7 +613,9 @@ impl<S: io::Read + io::Write> TlsStream<S> {
     pub fn get_mut(&mut self) -> &mut S {
         self.0.get_mut()
     }
+}
 
+impl<S: io::Read + io::Write> TlsStream<S> {
     /// Returns the number of bytes that can be read without resulting in any
     /// network calls.
     pub fn buffered_read_size(&self) -> Result<usize> {
